@@ -12,9 +12,10 @@ from typing import Tuple
 
 import jax
 import jax.lax as lax
-from flax import struct
 from jax import numpy as jnp
 
+from pgx.flax.struct import dataclass
+
 SHOT_COOL_DOWN = jnp.int8(5)
 ENEMY_MOVE_INTERVAL = jnp.int8(12)
 ENEMY_SHOT_INTERVAL = jnp.int8(10)
@@ -23,7 +24,7 @@ ZERO = jnp.int8(0)
 NINE = jnp.int8(9)
 
 
-@struct.dataclass
+@dataclass
 class State:
     pos: jnp.ndarray = jnp.int8(5)
     f_bullet_map: jnp.ndarray = jnp.zeros((10, 10), dtype=jnp.bool_)
