@@ -8,10 +8,10 @@ import numpy as np
 import svgwrite  # type: ignore
 
 from .animal_shogi import JaxAnimalShogiState as AnimalShogiState
-from .backgammon import BackgammonState
+from .backgammon import State as BackgammonState
 from .chess import ChessState
 from .contractbridgebidding import ContractBridgeBiddingState
-from .go import GoState
+from .go import State as GoState
 from .shogi import State as ShogiState
 from .suzume_jong import State as SuzumeJongState
 from .tic_tac_toe import State as TictactoeState
